`timescale 1ns / 1ps
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Design Name: PPWM
// Module Name: gate_net
// Target Devices: Spartan 6
// Description: 
//   Distributed as an artifact of ‘A Power to Pulse Width Modulation Sensor for Remote Power Analysis Attacks’
//   Copyright (c) 2022 B. Udugama, D. Jayasinghe, H. Saadat, A. Ignjatovic and S. Parameswaran
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

module gate_net(input clk, input delayed_in, input undelayed_in, output counter_value);	  
  	wire [1:0] delayed_path_conn;
  	wire [1:0] undelayed_path_conn;
	wire       FF_in;
	  
	  //Top LUT6_2 in Figure 5
	  (* s = "true" *) LUT6_2 #(
		 .INIT(64'b01010101_01010101_10101010_10101010_11110000_11110000_11110000_11110000)
	  ) LUT6_2_inst2 (
		 .O6(delayed_path_conn[1]),
		 .O5(undelayed_path_conn[0]),
		 .I0(delayed_path_conn[0]),
		 .I1(1'b0),
		 .I2(undelayed_in),
		 .I3(1'b0),
		 .I4(delayed_in),
		 .I5(1'b1)
	  );
	  
	  //Bottom LUT6_2 in Figure 5
	  (* s = "true" *) LUT6_2 #(
		 .INIT(64'b01010101_01010101_10101010_10101010_11110000_11110000_11110000_11110000)
	  ) LUT6_2_inst1 (
		 .O6(undelayed_path_conn[1]),
		 .O5(delayed_path_conn[0]),
		 .I0(undelayed_path_conn[0]),
		 .I1(1'b0),
		 .I2(delayed_in),
		 .I3(1'b0),
		 .I4(undelayed_in),
		 .I5(1'b1)
	  );
	  
	  // AND gate
	  (* s = "true" *) LUT6 #(
		 .INIT(64'b00000000_00000000_00000000_00000000_00000000_00000000_00000000_00001000)
	  ) LUT6_inst (
		 .O(FF_in),
		 .I0(delayed_path_conn[1]),
		 .I1(undelayed_path_conn[1]),
		 .I2(1'b0),
		 .I3(1'b0),
		 .I4(1'b0),
		 .I5(1'b0)
	  );
	  
	  (* s = "true" *) FDCE #(
		 .INIT(1'b0)
	  ) FDCE_inst (
		 .Q(counter_value),
		 .C(clk),
		 .CE(1'b1),
		 .CLR(FF_in),
		 .D(1'b1)
	  );

endmodule